@@ -9,10 +9,13 @@ import (
 	"errors"
 	"fmt"
 	"io"
+	"io/ioutil"
 	"os"
 	"runtime/debug"
 	"sync"
 
+	"github.com/evanw/esbuild/internal/fs"
+	"github.com/evanw/esbuild/internal/logger"
 	"github.com/evanw/esbuild/pkg/api"
 	"github.com/evanw/esbuild/pkg/cli"
 )
@@ -36,26 +39,29 @@ func runService() {
 		callbacks:       make(map[uint32]responseCallback),
 		outgoingPackets: make(chan outgoingPacket),
 	}
-	buffer := make([]byte, 4096)
+	buffer := make([]byte, 16*1024)
 	stream := []byte{}
 
-	// Write messages on a single goroutine so they aren't interleaved
+	// Write packets on a single goroutine so they aren't interleaved
 	waitGroup := &sync.WaitGroup{}
 	go func() {
 		for {
-			message, ok := <-service.outgoingPackets
+			packet, ok := <-service.outgoingPackets
 			if !ok {
-				break // No more messages
+				break // No more packets
 			}
-			os.Stdout.Write(message.bytes)
+			os.Stdout.Write(packet.bytes)
 
 			// Only signal that this request is done when it has actually been written
-			if message.isFinal {
+			if packet.isFinal {
 				waitGroup.Done()
 			}
 		}
 	}()
 
+	// The protocol always starts with the version
+	os.Stdout.Write(append(writeUint32(nil, uint32(len(esbuildVersion))), esbuildVersion...))
+
 	for {
 		// Read more data from stdin
 		n, err := os.Stdin.Read(buffer)
@@ -67,20 +73,20 @@ func runService() {
 		}
 		stream = append(stream, buffer[:n]...)
 
-		// Process all complete (i.e. not partial) messages
+		// Process all complete (i.e. not partial) packets
 		bytes := stream
 		for {
-			message, afterMessage, ok := readLengthPrefixedSlice(bytes)
+			packet, afterPacket, ok := readLengthPrefixedSlice(bytes)
 			if !ok {
 				break
 			}
-			bytes = afterMessage
+			bytes = afterPacket
 
 			// Clone the input and run it on another goroutine
-			clone := append([]byte{}, message...)
+			clone := append([]byte{}, packet...)
 			waitGroup.Add(1)
 			go func() {
-				if result := service.handleIncomingMessage(clone); result != nil {
+				if result := service.handleIncomingPacket(clone); result != nil {
 					service.outgoingPackets <- outgoingPacket{bytes: result, isFinal: true}
 				} else {
 					waitGroup.Done()
@@ -88,7 +94,7 @@ func runService() {
 			}()
 		}
 
-		// Move the remaining partial message to the end to avoid reallocating
+		// Move the remaining partial packet to the end to avoid reallocating
 		stream = append(stream[:0], bytes...)
 	}
 
@@ -121,7 +127,7 @@ func (service *serviceType) sendRequest(request interface{}) interface{} {
 	return <-result
 }
 
-func (service *serviceType) handleIncomingMessage(bytes []byte) (result []byte) {
+func (service *serviceType) handleIncomingPacket(bytes []byte) (result []byte) {
 	p, ok := decodePacket(bytes)
 	if !ok {
 		return nil
@@ -141,19 +147,32 @@ func (service *serviceType) handleIncomingMessage(bytes []byte) (result []byte)
 		}()
 
 		// Handle the request
-		data := p.value.([]interface{})
-		switch data[0] {
+		request := p.value.(map[string]interface{})
+		command := request["command"].(string)
+		switch command {
 		case "build":
-			return service.handleBuildRequest(p.id, data[1].(map[string]interface{}))
+			return service.handleBuildRequest(p.id, request)
 
 		case "transform":
-			return service.handleTransformRequest(p.id, data[1].(map[string]interface{}))
+			return service.handleTransformRequest(p.id, request)
+
+		case "error":
+			// This just exists so that errors during JavaScript API setup get printed
+			// nicely to the console. This matters if the JavaScript API setup code
+			// swallows thrown errors. We still want to be able to see the error.
+			flags := decodeStringArray(request["flags"].([]interface{}))
+			msg := decodeMessageToPrivate(request["error"].(map[string]interface{}))
+			logger.PrintMessageToStderr(flags, msg)
+			return encodePacket(packet{
+				id:    p.id,
+				value: make(map[string]interface{}),
+			})
 
 		default:
 			return encodePacket(packet{
 				id: p.id,
 				value: map[string]interface{}{
-					"error": fmt.Sprintf("Invalid command: %s", p.value),
+					"error": fmt.Sprintf("Invalid command: %s", command),
 				},
 			})
 		}
@@ -171,32 +190,34 @@ func (service *serviceType) handleIncomingMessage(bytes []byte) (result []byte)
 	return nil
 }
 
+func encodeErrorPacket(id uint32, err error) []byte {
+	return encodePacket(packet{
+		id: id,
+		value: map[string]interface{}{
+			"error": err.Error(),
+		},
+	})
+}
+
 func (service *serviceType) handleBuildRequest(id uint32, request map[string]interface{}) []byte {
 	write := request["write"].(bool)
 	flags := decodeStringArray(request["flags"].([]interface{}))
-	stdin, hasStdin := request["stdin"].(string)
-	resolveDir, hasResolveDir := request["resolveDir"].(string)
 
 	options, err := cli.ParseBuildOptions(flags)
 	if err == nil && write && options.Outfile == "" && options.Outdir == "" {
 		err = errors.New("Either provide \"outfile\" or set \"write\" to false")
 	}
 	if err != nil {
-		return encodePacket(packet{
-			id: id,
-			value: map[string]interface{}{
-				"error": err.Error(),
-			},
-		})
+		return encodeErrorPacket(id, err)
 	}
 
 	// Optionally allow input from the stdin channel
-	if hasStdin {
+	if stdin, ok := request["stdin"].(string); ok {
 		if options.Stdin == nil {
 			options.Stdin = &api.StdinOptions{}
 		}
 		options.Stdin.Contents = stdin
-		if hasResolveDir {
+		if resolveDir, ok := request["resolveDir"].(string); ok {
 			options.Stdin.ResolveDir = resolveDir
 		}
 	}
@@ -220,27 +241,64 @@ func (service *serviceType) handleBuildRequest(id uint32, request map[string]int
 }
 
 func (service *serviceType) handleTransformRequest(id uint32, request map[string]interface{}) []byte {
+	inputFS := request["inputFS"].(bool)
 	input := request["input"].(string)
 	flags := decodeStringArray(request["flags"].([]interface{}))
 
 	options, err := cli.ParseTransformOptions(flags)
 	if err != nil {
-		return encodePacket(packet{
-			id: id,
-			value: map[string]interface{}{
-				"error": err.Error(),
-			},
-		})
+		return encodeErrorPacket(id, err)
+	}
+
+	transformInput := input
+	if inputFS {
+		fs.BeforeFileOpen()
+		bytes, err := ioutil.ReadFile(input)
+		fs.AfterFileClose()
+		if err == nil {
+			err = os.Remove(input)
+		}
+		if err != nil {
+			return encodeErrorPacket(id, err)
+		}
+		transformInput = string(bytes)
+	}
+
+	result := api.Transform(transformInput, options)
+	jsFS := false
+	jsSourceMapFS := false
+
+	if inputFS && len(result.JS) > 0 {
+		file := input + ".js"
+		fs.BeforeFileOpen()
+		if err := ioutil.WriteFile(file, result.JS, 0644); err == nil {
+			result.JS = []byte(file)
+			jsFS = true
+		}
+		fs.AfterFileClose()
+	}
+
+	if inputFS && len(result.JSSourceMap) > 0 {
+		file := input + ".map"
+		fs.BeforeFileOpen()
+		if err := ioutil.WriteFile(file, result.JSSourceMap, 0644); err == nil {
+			result.JSSourceMap = []byte(file)
+			jsSourceMapFS = true
+		}
+		fs.AfterFileClose()
 	}
 
-	result := api.Transform(input, options)
 	return encodePacket(packet{
 		id: id,
 		value: map[string]interface{}{
-			"errors":      encodeMessages(result.Errors),
-			"warnings":    encodeMessages(result.Warnings),
-			"js":          string(result.JS),
-			"jsSourceMap": string(result.JSSourceMap),
+			"errors":   encodeMessages(result.Errors),
+			"warnings": encodeMessages(result.Warnings),
+
+			"jsFS": jsFS,
+			"js":   string(result.JS),
+
+			"jsSourceMapFS": jsSourceMapFS,
+			"jsSourceMap":   string(result.JSSourceMap),
 		},
 	})
 }
@@ -287,3 +345,22 @@ func encodeMessages(msgs []api.Message) []interface{} {
 	}
 	return values
 }
+
+func decodeMessageToPrivate(obj map[string]interface{}) logger.Msg {
+	msg := logger.Msg{Text: obj["text"].(string)}
+
+	// Some messages won't have a location
+	loc := obj["location"]
+	if loc != nil {
+		loc := loc.(map[string]interface{})
+		msg.Location = &logger.MsgLocation{
+			File:     loc["file"].(string),
+			Line:     loc["line"].(int),
+			Column:   loc["column"].(int),
+			Length:   loc["length"].(int),
+			LineText: loc["lineText"].(string),
+		}
+	}
+
+	return msg
+}
