@@ -6,8 +6,12 @@ import (
 	"github.com/evanw/esbuild/internal/config"
 )
 
+var splitting_suite = suite{
+	name: "splitting",
+}
+
 func TestSplittingSharedES6IntoES6(t *testing.T) {
-	expectBundled(t, bundled{
+	splitting_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/a.js": `
 				import {foo} from "./shared.js"
@@ -21,39 +25,16 @@ func TestSplittingSharedES6IntoES6(t *testing.T) {
 		},
 		entryPaths: []string{"/a.js", "/b.js"},
 		options: config.Options{
-			IsBundling:    true,
+			Mode:          config.ModeBundle,
 			CodeSplitting: true,
 			OutputFormat:  config.FormatESModule,
 			AbsOutputDir:  "/out",
 		},
-		expected: map[string]string{
-			"/out/a.js": `import {
-  foo
-} from "./chunk.xL6KqlYO.js";
-
-// /a.js
-console.log(foo);
-`,
-			"/out/b.js": `import {
-  foo
-} from "./chunk.xL6KqlYO.js";
-
-// /b.js
-console.log(foo);
-`,
-			"/out/chunk.xL6KqlYO.js": `// /shared.js
-let foo = 123;
-
-export {
-  foo
-};
-`,
-		},
 	})
 }
 
 func TestSplittingSharedCommonJSIntoES6(t *testing.T) {
-	expectBundled(t, bundled{
+	splitting_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/a.js": `
 				const {foo} = require("./shared.js")
@@ -67,43 +48,16 @@ func TestSplittingSharedCommonJSIntoES6(t *testing.T) {
 		},
 		entryPaths: []string{"/a.js", "/b.js"},
 		options: config.Options{
-			IsBundling:    true,
+			Mode:          config.ModeBundle,
 			CodeSplitting: true,
 			OutputFormat:  config.FormatESModule,
 			AbsOutputDir:  "/out",
 		},
-		expected: map[string]string{
-			"/out/a.js": `import {
-  require_shared
-} from "./chunk.xL6KqlYO.js";
-
-// /a.js
-const {foo} = require_shared();
-console.log(foo);
-`,
-			"/out/b.js": `import {
-  require_shared
-} from "./chunk.xL6KqlYO.js";
-
-// /b.js
-const {foo: foo2} = require_shared();
-console.log(foo2);
-`,
-			"/out/chunk.xL6KqlYO.js": `// /shared.js
-var require_shared = __commonJS((exports) => {
-  exports.foo = 123;
-});
-
-export {
-  require_shared
-};
-`,
-		},
 	})
 }
 
 func TestSplittingDynamicES6IntoES6(t *testing.T) {
-	expectBundled(t, bundled{
+	splitting_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/entry.js": `
 				import("./foo.js").then(({bar}) => console.log(bar))
@@ -114,27 +68,16 @@ func TestSplittingDynamicES6IntoES6(t *testing.T) {
 		},
 		entryPaths: []string{"/entry.js"},
 		options: config.Options{
-			IsBundling:    true,
+			Mode:          config.ModeBundle,
 			CodeSplitting: true,
 			OutputFormat:  config.FormatESModule,
 			AbsOutputDir:  "/out",
 		},
-		expected: map[string]string{
-			"/out/entry.js": `// /entry.js
-import("./foo.js").then(({bar: bar2}) => console.log(bar2));
-`,
-			"/out/foo.js": `// /foo.js
-let bar = 123;
-export {
-  bar
-};
-`,
-		},
 	})
 }
 
 func TestSplittingDynamicCommonJSIntoES6(t *testing.T) {
-	expectBundled(t, bundled{
+	splitting_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/entry.js": `
 				import("./foo.js").then(({default: {bar}}) => console.log(bar))
@@ -145,27 +88,16 @@ func TestSplittingDynamicCommonJSIntoES6(t *testing.T) {
 		},
 		entryPaths: []string{"/entry.js"},
 		options: config.Options{
-			IsBundling:    true,
+			Mode:          config.ModeBundle,
 			CodeSplitting: true,
 			OutputFormat:  config.FormatESModule,
 			AbsOutputDir:  "/out",
 		},
-		expected: map[string]string{
-			"/out/entry.js": `// /entry.js
-import("./foo.js").then(({default: {bar}}) => console.log(bar));
-`,
-			"/out/foo.js": `// /foo.js
-var require_foo = __commonJS((exports) => {
-  exports.bar = 123;
-});
-export default require_foo();
-`,
-		},
 	})
 }
 
 func TestSplittingDynamicAndNotDynamicES6IntoES6(t *testing.T) {
-	expectBundled(t, bundled{
+	splitting_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/entry.js": `
 				import {bar as a} from "./foo.js"
@@ -177,41 +109,16 @@ func TestSplittingDynamicAndNotDynamicES6IntoES6(t *testing.T) {
 		},
 		entryPaths: []string{"/entry.js"},
 		options: config.Options{
-			IsBundling:    true,
+			Mode:          config.ModeBundle,
 			CodeSplitting: true,
 			OutputFormat:  config.FormatESModule,
 			AbsOutputDir:  "/out",
 		},
-		expected: map[string]string{
-			"/out/entry.js": `import {
-  bar
-} from "./chunk.-fk8OGuR.js";
-
-// /entry.js
-import("./foo.js").then(({bar: b}) => console.log(bar, b));
-`,
-			"/out/foo.js": `import {
-  bar
-} from "./chunk.-fk8OGuR.js";
-
-// /foo.js
-export {
-  bar
-};
-`,
-			"/out/chunk.-fk8OGuR.js": `// /foo.js
-let bar = 123;
-
-export {
-  bar
-};
-`,
-		},
 	})
 }
 
 func TestSplittingDynamicAndNotDynamicCommonJSIntoES6(t *testing.T) {
-	expectBundled(t, bundled{
+	splitting_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/entry.js": `
 				import {bar as a} from "./foo.js"
@@ -223,42 +130,16 @@ func TestSplittingDynamicAndNotDynamicCommonJSIntoES6(t *testing.T) {
 		},
 		entryPaths: []string{"/entry.js"},
 		options: config.Options{
-			IsBundling:    true,
+			Mode:          config.ModeBundle,
 			CodeSplitting: true,
 			OutputFormat:  config.FormatESModule,
 			AbsOutputDir:  "/out",
 		},
-		expected: map[string]string{
-			"/out/entry.js": `import {
-  require_foo
-} from "./chunk.-fk8OGuR.js";
-
-// /entry.js
-const foo = __toModule(require_foo());
-import("./foo.js").then(({default: {bar: b}}) => console.log(foo.bar, b));
-`,
-			"/out/foo.js": `import {
-  require_foo
-} from "./chunk.-fk8OGuR.js";
-
-// /foo.js
-export default require_foo();
-`,
-			"/out/chunk.-fk8OGuR.js": `// /foo.js
-var require_foo = __commonJS((exports) => {
-  exports.bar = 123;
-});
-
-export {
-  require_foo
-};
-`,
-		},
 	})
 }
 
 func TestSplittingAssignToLocal(t *testing.T) {
-	expectBundled(t, bundled{
+	splitting_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/a.js": `
 				import {foo, setFoo} from "./shared.js"
@@ -278,45 +159,16 @@ func TestSplittingAssignToLocal(t *testing.T) {
 		},
 		entryPaths: []string{"/a.js", "/b.js"},
 		options: config.Options{
-			IsBundling:    true,
+			Mode:          config.ModeBundle,
 			CodeSplitting: true,
 			OutputFormat:  config.FormatESModule,
 			AbsOutputDir:  "/out",
 		},
-		expected: map[string]string{
-			"/out/a.js": `import {
-  foo,
-  setFoo
-} from "./chunk.xL6KqlYO.js";
-
-// /a.js
-setFoo(123);
-console.log(foo);
-`,
-			"/out/b.js": `import {
-  foo
-} from "./chunk.xL6KqlYO.js";
-
-// /b.js
-console.log(foo);
-`,
-			"/out/chunk.xL6KqlYO.js": `// /shared.js
-let foo;
-function setFoo(value) {
-  foo = value;
-}
-
-export {
-  foo,
-  setFoo
-};
-`,
-		},
 	})
 }
 
 func TestSplittingSideEffectsWithoutDependencies(t *testing.T) {
-	expectBundled(t, bundled{
+	splitting_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/a.js": `
 				import {a} from "./shared.js"
@@ -334,37 +186,16 @@ func TestSplittingSideEffectsWithoutDependencies(t *testing.T) {
 		},
 		entryPaths: []string{"/a.js", "/b.js"},
 		options: config.Options{
-			IsBundling:    true,
+			Mode:          config.ModeBundle,
 			CodeSplitting: true,
 			OutputFormat:  config.FormatESModule,
 			AbsOutputDir:  "/out",
 		},
-		expected: map[string]string{
-			"/out/a.js": `import "./chunk.xL6KqlYO.js";
-
-// /shared.js
-let a = 1;
-
-// /a.js
-console.log(a);
-`,
-			"/out/b.js": `import "./chunk.xL6KqlYO.js";
-
-// /shared.js
-let b = 2;
-
-// /b.js
-console.log(b);
-`,
-			"/out/chunk.xL6KqlYO.js": `// /shared.js
-console.log("side effect");
-`,
-		},
 	})
 }
 
 func TestSplittingNestedDirectories(t *testing.T) {
-	expectBundled(t, bundled{
+	splitting_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/Users/user/project/src/pages/pageA/page.js": `
 				import x from "../shared.js"
@@ -383,39 +214,16 @@ func TestSplittingNestedDirectories(t *testing.T) {
 			"/Users/user/project/src/pages/pageB/page.js",
 		},
 		options: config.Options{
-			IsBundling:    true,
+			Mode:          config.ModeBundle,
 			CodeSplitting: true,
 			OutputFormat:  config.FormatESModule,
 			AbsOutputDir:  "/Users/user/project/out",
 		},
-		expected: map[string]string{
-			"/Users/user/project/out/pageA/page.js": `import {
-  shared_default
-} from "../chunk.UcWke4C2.js";
-
-// /Users/user/project/src/pages/pageA/page.js
-console.log(shared_default);
-`,
-			"/Users/user/project/out/pageB/page.js": `import {
-  shared_default
-} from "../chunk.UcWke4C2.js";
-
-// /Users/user/project/src/pages/pageB/page.js
-console.log(-shared_default);
-`,
-			"/Users/user/project/out/chunk.UcWke4C2.js": `// /Users/user/project/src/pages/shared.js
-var shared_default = 123;
-
-export {
-  shared_default
-};
-`,
-		},
 	})
 }
 
 func TestSplittingCircularReferenceIssue251(t *testing.T) {
-	expectBundled(t, bundled{
+	splitting_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/a.js": `
 				export * from './b.js';
@@ -428,51 +236,16 @@ func TestSplittingCircularReferenceIssue251(t *testing.T) {
 		},
 		entryPaths: []string{"/a.js", "/b.js"},
 		options: config.Options{
-			IsBundling:    true,
+			Mode:          config.ModeBundle,
 			CodeSplitting: true,
 			OutputFormat:  config.FormatESModule,
 			AbsOutputDir:  "/out",
 		},
-		expected: map[string]string{
-			"/out/a.js": `import {
-  p,
-  q
-} from "./chunk.xL6KqlYO.js";
-
-// /a.js
-export {
-  p,
-  q
-};
-`,
-			"/out/b.js": `import {
-  p,
-  q
-} from "./chunk.xL6KqlYO.js";
-
-// /b.js
-export {
-  p,
-  q
-};
-`,
-			"/out/chunk.xL6KqlYO.js": `// /b.js
-var q = 6;
-
-// /a.js
-var p = 5;
-
-export {
-  p,
-  q
-};
-`,
-		},
 	})
 }
 
 func TestSplittingMissingLazyExport(t *testing.T) {
-	expectBundled(t, bundled{
+	splitting_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/a.js": `
 				import {foo} from './common.js'
@@ -494,45 +267,18 @@ func TestSplittingMissingLazyExport(t *testing.T) {
 		},
 		entryPaths: []string{"/a.js", "/b.js"},
 		options: config.Options{
-			IsBundling:    true,
+			Mode:          config.ModeBundle,
 			CodeSplitting: true,
 			OutputFormat:  config.FormatESModule,
 			AbsOutputDir:  "/out",
 		},
-		expected: map[string]string{
-			"/out/a.js": `import "./chunk.xL6KqlYO.js";
-
-// /empty.js
-const empty_exports = {};
-
-// /common.js
-function foo() {
-  return [empty_exports, void 0];
-}
-
-// /a.js
-console.log(foo());
+		expectedCompileLog: `/common.js: warning: No matching export for import "missing"
 `,
-			"/out/b.js": `import "./chunk.xL6KqlYO.js";
-
-// /common.js
-function bar() {
-  return [void 0];
-}
-
-// /b.js
-console.log(bar());
-`,
-			"/out/chunk.xL6KqlYO.js": `// /empty.js
-
-// /common.js
-`,
-		},
 	})
 }
 
 func TestSplittingReExportIssue273(t *testing.T) {
-	expectBundled(t, bundled{
+	splitting_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/a.js": `
 				export const a = 1
@@ -543,43 +289,16 @@ func TestSplittingReExportIssue273(t *testing.T) {
 		},
 		entryPaths: []string{"/a.js", "/b.js"},
 		options: config.Options{
-			IsBundling:    true,
+			Mode:          config.ModeBundle,
 			CodeSplitting: true,
 			OutputFormat:  config.FormatESModule,
 			AbsOutputDir:  "/out",
 		},
-		expected: map[string]string{
-			"/out/a.js": `import {
-  a
-} from "./chunk.xL6KqlYO.js";
-
-// /a.js
-export {
-  a
-};
-`,
-			"/out/b.js": `import {
-  a
-} from "./chunk.xL6KqlYO.js";
-
-// /b.js
-export {
-  a
-};
-`,
-			"/out/chunk.xL6KqlYO.js": `// /a.js
-const a = 1;
-
-export {
-  a
-};
-`,
-		},
 	})
 }
 
 func TestSplittingDynamicImportIssue272(t *testing.T) {
-	expectBundled(t, bundled{
+	splitting_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/a.js": `
 				import('./b')
@@ -590,27 +309,16 @@ func TestSplittingDynamicImportIssue272(t *testing.T) {
 		},
 		entryPaths: []string{"/a.js", "/b.js"},
 		options: config.Options{
-			IsBundling:    true,
+			Mode:          config.ModeBundle,
 			CodeSplitting: true,
 			OutputFormat:  config.FormatESModule,
 			AbsOutputDir:  "/out",
 		},
-		expected: map[string]string{
-			"/out/a.js": `// /a.js
-import("./b.js");
-`,
-			"/out/b.js": `// /b.js
-var b_default = 1;
-export {
-  b_default as default
-};
-`,
-		},
 	})
 }
 
 func TestSplittingDynamicImportOutsideSourceTreeIssue264(t *testing.T) {
-	expectBundled(t, bundled{
+	splitting_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/Users/user/project/src/entry1.js": `
 				import('package')
@@ -624,21 +332,126 @@ func TestSplittingDynamicImportOutsideSourceTreeIssue264(t *testing.T) {
 		},
 		entryPaths: []string{"/Users/user/project/src/entry1.js", "/Users/user/project/src/entry2.js"},
 		options: config.Options{
-			IsBundling:    true,
+			Mode:          config.ModeBundle,
 			CodeSplitting: true,
 			OutputFormat:  config.FormatESModule,
 			AbsOutputDir:  "/out",
 		},
-		expected: map[string]string{
-			"/out/src/entry1.js": `// /Users/user/project/src/entry1.js
-import("../node_modules/package/index.js");
-`,
-			"/out/src/entry2.js": `// /Users/user/project/src/entry2.js
-import("../node_modules/package/index.js");
-`,
-			"/out/node_modules/package/index.js": `// /Users/user/project/node_modules/package/index.js
-console.log("imported");
-`,
+	})
+}
+
+func TestSplittingCrossChunkAssignmentDependencies(t *testing.T) {
+	splitting_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/a.js": `
+				import {setValue} from './shared'
+				setValue(123)
+			`,
+			"/b.js": `
+				import './shared'
+			`,
+			"/shared.js": `
+				var observer;
+				var value;
+				export function setObserver(cb) {
+					observer = cb;
+				}
+				export function getValue() {
+					return value;
+				}
+				export function setValue(next) {
+					value = next;
+					if (observer) observer();
+				}
+				sideEffects(getValue);
+			`,
+		},
+		entryPaths: []string{"/a.js", "/b.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			CodeSplitting: true,
+			OutputFormat:  config.FormatESModule,
+			AbsOutputDir:  "/out",
+		},
+	})
+}
+
+func TestSplittingCrossChunkAssignmentDependenciesRecursive(t *testing.T) {
+	splitting_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/a.js": `
+				import { setX } from './x'
+				setX()
+			`,
+			"/b.js": `
+				import { setZ } from './z'
+				setZ()
+			`,
+			"/c.js": `
+				import { setX2 } from './x'
+				import { setY2 } from './y'
+				import { setZ2 } from './z'
+				setX2();
+				setY2();
+				setZ2();
+			`,
+			"/x.js": `
+				let _x
+				export function setX(v) { _x = v }
+				export function setX2(v) { _x = v }
+			`,
+			"/y.js": `
+				import { setX } from './x'
+				let _y
+				export function setY(v) { _y = v }
+				export function setY2(v) { setX(v); _y = v }
+			`,
+			"/z.js": `
+				import { setY } from './y'
+				let _z
+				export function setZ(v) { _z = v }
+				export function setZ2(v) { setY(v); _z = v }
+			`,
+		},
+		entryPaths: []string{"/a.js", "/b.js", "/c.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			CodeSplitting: true,
+			OutputFormat:  config.FormatESModule,
+			AbsOutputDir:  "/out",
+		},
+	})
+}
+
+func TestSplittingDuplicateChunkCollision(t *testing.T) {
+	splitting_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/a.js": `
+				import "./ab"
+			`,
+			"/b.js": `
+				import "./ab"
+			`,
+			"/c.js": `
+				import "./cd"
+			`,
+			"/d.js": `
+				import "./cd"
+			`,
+			"/ab.js": `
+				console.log(123)
+			`,
+			"/cd.js": `
+				console.log(123)
+			`,
+		},
+		entryPaths: []string{"/a.js", "/b.js", "/c.js", "/d.js"},
+		options: config.Options{
+			Mode:             config.ModeBundle,
+			CodeSplitting:    true,
+			RemoveWhitespace: true,
+			OutputFormat:     config.FormatESModule,
+			AbsOutputDir:     "/out",
 		},
 	})
 }
