@@ -6,8 +6,12 @@ import (
 	"github.com/evanw/esbuild/internal/config"
 )
 
+var tsconfig_suite = suite{
+	name: "tsconfig",
+}
+
 func TestTsConfigPaths(t *testing.T) {
-	expectBundled(t, bundled{
+	tsconfig_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/Users/user/project/entry.ts": `
 				import baseurl_dot from './baseurl_dot'
@@ -119,75 +123,14 @@ func TestTsConfigPaths(t *testing.T) {
 		},
 		entryPaths: []string{"/Users/user/project/entry.ts"},
 		options: config.Options{
-			IsBundling:    true,
+			Mode:          config.ModeBundle,
 			AbsOutputFile: "/Users/user/project/out.js",
 		},
-		expected: map[string]string{
-			"/Users/user/project/out.js": `// /Users/user/project/baseurl_dot/test0-success.ts
-var test0_success_default = "test0-success";
-
-// /Users/user/project/baseurl_dot/test1-success.ts
-var test1_success_default = "test1-success";
-
-// /Users/user/project/baseurl_dot/test2-success/foo.ts
-var foo_default = "test2-success";
-
-// /Users/user/project/baseurl_dot/test3-success.ts
-var test3_success_default = "test3-success";
-
-// /Users/user/project/baseurl_dot/test4-first/foo.ts
-var foo_default2 = "test4-success";
-
-// /Users/user/project/baseurl_dot/test5-second/foo.ts
-var foo_default3 = "test5-success";
-
-// /Users/user/project/baseurl_dot/index.ts
-var baseurl_dot_default = {
-  test0: test0_success_default,
-  test1: test1_success_default,
-  test2: foo_default,
-  test3: test3_success_default,
-  test4: foo_default2,
-  test5: foo_default3
-};
-
-// /Users/user/project/baseurl_nested/nested/test0-success.ts
-var test0_success_default2 = "test0-success";
-
-// /Users/user/project/baseurl_nested/nested/test1-success.ts
-var test1_success_default2 = "test1-success";
-
-// /Users/user/project/baseurl_nested/nested/test2-success/foo.ts
-var foo_default4 = "test2-success";
-
-// /Users/user/project/baseurl_nested/nested/test3-success.ts
-var test3_success_default2 = "test3-success";
-
-// /Users/user/project/baseurl_nested/nested/test4-first/foo.ts
-var foo_default5 = "test4-success";
-
-// /Users/user/project/baseurl_nested/nested/test5-second/foo.ts
-var foo_default6 = "test5-success";
-
-// /Users/user/project/baseurl_nested/index.ts
-var baseurl_nested_default = {
-  test0: test0_success_default2,
-  test1: test1_success_default2,
-  test2: foo_default4,
-  test3: test3_success_default2,
-  test4: foo_default5,
-  test5: foo_default6
-};
-
-// /Users/user/project/entry.ts
-console.log(baseurl_dot_default, baseurl_nested_default);
-`,
-		},
 	})
 }
 
 func TestTsConfigJSX(t *testing.T) {
-	expectBundled(t, bundled{
+	tsconfig_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/Users/user/project/entry.tsx": `
 				console.log(<><div/><div/></>)
@@ -203,19 +146,14 @@ func TestTsConfigJSX(t *testing.T) {
 		},
 		entryPaths: []string{"/Users/user/project/entry.tsx"},
 		options: config.Options{
-			IsBundling:    true,
+			Mode:          config.ModeBundle,
 			AbsOutputFile: "/Users/user/project/out.js",
 		},
-		expected: map[string]string{
-			"/Users/user/project/out.js": `// /Users/user/project/entry.tsx
-console.log(/* @__PURE__ */ R.c(R.F, null, /* @__PURE__ */ R.c("div", null), /* @__PURE__ */ R.c("div", null)));
-`,
-		},
 	})
 }
 
 func TestTsConfigNestedJSX(t *testing.T) {
-	expectBundled(t, bundled{
+	tsconfig_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/Users/user/project/entry.ts": `
 				import factory from './factory'
@@ -257,28 +195,14 @@ func TestTsConfigNestedJSX(t *testing.T) {
 		},
 		entryPaths: []string{"/Users/user/project/entry.ts"},
 		options: config.Options{
-			IsBundling:    true,
+			Mode:          config.ModeBundle,
 			AbsOutputFile: "/Users/user/project/out.js",
 		},
-		expected: map[string]string{
-			"/Users/user/project/out.js": `// /Users/user/project/factory/index.tsx
-var factory_default = /* @__PURE__ */ h(React.Fragment, null, /* @__PURE__ */ h("div", null), /* @__PURE__ */ h("div", null));
-
-// /Users/user/project/fragment/index.tsx
-var fragment_default = /* @__PURE__ */ React.createElement(a.b, null, /* @__PURE__ */ React.createElement("div", null), /* @__PURE__ */ React.createElement("div", null));
-
-// /Users/user/project/both/index.tsx
-var both_default = /* @__PURE__ */ R.c(R.F, null, /* @__PURE__ */ R.c("div", null), /* @__PURE__ */ R.c("div", null));
-
-// /Users/user/project/entry.ts
-console.log(factory_default, fragment_default, both_default);
-`,
-		},
 	})
 }
 
 func TestTsconfigJsonBaseUrl(t *testing.T) {
-	expectBundled(t, bundled{
+	tsconfig_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/Users/user/project/src/app/entry.js": `
 				import fn from 'lib/util'
@@ -299,27 +223,14 @@ func TestTsconfigJsonBaseUrl(t *testing.T) {
 		},
 		entryPaths: []string{"/Users/user/project/src/app/entry.js"},
 		options: config.Options{
-			IsBundling:    true,
+			Mode:          config.ModeBundle,
 			AbsOutputFile: "/Users/user/project/out.js",
 		},
-		expected: map[string]string{
-			"/Users/user/project/out.js": `// /Users/user/project/src/lib/util.js
-var require_util = __commonJS((exports, module) => {
-  module.exports = function() {
-    return 123;
-  };
-});
-
-// /Users/user/project/src/app/entry.js
-const util = __toModule(require_util());
-console.log(util.default());
-`,
-		},
 	})
 }
 
 func TestJsconfigJsonBaseUrl(t *testing.T) {
-	expectBundled(t, bundled{
+	tsconfig_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/Users/user/project/src/app/entry.js": `
 				import fn from 'lib/util'
@@ -340,27 +251,14 @@ func TestJsconfigJsonBaseUrl(t *testing.T) {
 		},
 		entryPaths: []string{"/Users/user/project/src/app/entry.js"},
 		options: config.Options{
-			IsBundling:    true,
+			Mode:          config.ModeBundle,
 			AbsOutputFile: "/Users/user/project/out.js",
 		},
-		expected: map[string]string{
-			"/Users/user/project/out.js": `// /Users/user/project/src/lib/util.js
-var require_util = __commonJS((exports, module) => {
-  module.exports = function() {
-    return 123;
-  };
-});
-
-// /Users/user/project/src/app/entry.js
-const util = __toModule(require_util());
-console.log(util.default());
-`,
-		},
 	})
 }
 
 func TestTsconfigJsonCommentAllowed(t *testing.T) {
-	expectBundled(t, bundled{
+	tsconfig_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/Users/user/project/src/app/entry.js": `
 				import fn from 'lib/util'
@@ -382,27 +280,14 @@ func TestTsconfigJsonCommentAllowed(t *testing.T) {
 		},
 		entryPaths: []string{"/Users/user/project/src/app/entry.js"},
 		options: config.Options{
-			IsBundling:    true,
+			Mode:          config.ModeBundle,
 			AbsOutputFile: "/Users/user/project/out.js",
 		},
-		expected: map[string]string{
-			"/Users/user/project/out.js": `// /Users/user/project/src/lib/util.js
-var require_util = __commonJS((exports, module) => {
-  module.exports = function() {
-    return 123;
-  };
-});
-
-// /Users/user/project/src/app/entry.js
-const util = __toModule(require_util());
-console.log(util.default());
-`,
-		},
 	})
 }
 
 func TestTsconfigJsonTrailingCommaAllowed(t *testing.T) {
-	expectBundled(t, bundled{
+	tsconfig_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/Users/user/project/src/app/entry.js": `
 				import fn from 'lib/util'
@@ -423,27 +308,14 @@ func TestTsconfigJsonTrailingCommaAllowed(t *testing.T) {
 		},
 		entryPaths: []string{"/Users/user/project/src/app/entry.js"},
 		options: config.Options{
-			IsBundling:    true,
+			Mode:          config.ModeBundle,
 			AbsOutputFile: "/Users/user/project/out.js",
 		},
-		expected: map[string]string{
-			"/Users/user/project/out.js": `// /Users/user/project/src/lib/util.js
-var require_util = __commonJS((exports, module) => {
-  module.exports = function() {
-    return 123;
-  };
-});
-
-// /Users/user/project/src/app/entry.js
-const util = __toModule(require_util());
-console.log(util.default());
-`,
-		},
 	})
 }
 
 func TestTsconfigJsonExtends(t *testing.T) {
-	expectBundled(t, bundled{
+	tsconfig_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/entry.jsx": `
 				console.log(<div/>, <></>)
@@ -467,19 +339,14 @@ func TestTsconfigJsonExtends(t *testing.T) {
 		},
 		entryPaths: []string{"/entry.jsx"},
 		options: config.Options{
-			IsBundling:    true,
+			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.js",
 		},
-		expected: map[string]string{
-			"/out.js": `// /entry.jsx
-console.log(/* @__PURE__ */ baseFactory("div", null), /* @__PURE__ */ baseFactory(derivedFragment, null));
-`,
-		},
 	})
 }
 
 func TestTsconfigJsonExtendsThreeLevels(t *testing.T) {
-	expectBundled(t, bundled{
+	tsconfig_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/entry.jsx": `
 				console.log(<div/>, <></>)
@@ -508,19 +375,14 @@ func TestTsconfigJsonExtendsThreeLevels(t *testing.T) {
 		},
 		entryPaths: []string{"/entry.jsx"},
 		options: config.Options{
-			IsBundling:    true,
+			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.js",
 		},
-		expected: map[string]string{
-			"/out.js": `// /entry.jsx
-console.log(/* @__PURE__ */ baseFactory("div", null), /* @__PURE__ */ baseFactory(derivedFragment, null));
-`,
-		},
 	})
 }
 
 func TestTsconfigJsonExtendsLoop(t *testing.T) {
-	expectBundled(t, bundled{
+	tsconfig_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/entry.js": `
 				console.log(123)
@@ -538,20 +400,15 @@ func TestTsconfigJsonExtendsLoop(t *testing.T) {
 		},
 		entryPaths: []string{"/entry.js"},
 		options: config.Options{
-			IsBundling:    true,
+			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.js",
 		},
 		expectedScanLog: "/base.json: warning: Base config file \"./tsconfig\" forms cycle\n",
-		expected: map[string]string{
-			"/out.js": `// /entry.js
-console.log(123);
-`,
-		},
 	})
 }
 
 func TestTsconfigJsonExtendsPackage(t *testing.T) {
-	expectBundled(t, bundled{
+	tsconfig_suite.expectBundled(t, bundled{
 		files: map[string]string{
 			"/Users/user/project/src/app/entry.jsx": `
 				console.log(<div/>)
@@ -571,13 +428,162 @@ func TestTsconfigJsonExtendsPackage(t *testing.T) {
 		},
 		entryPaths: []string{"/Users/user/project/src/app/entry.jsx"},
 		options: config.Options{
-			IsBundling:    true,
+			Mode:          config.ModeBundle,
 			AbsOutputFile: "/Users/user/project/out.js",
 		},
-		expected: map[string]string{
-			"/Users/user/project/out.js": `// /Users/user/project/src/app/entry.jsx
-console.log(/* @__PURE__ */ worked("div", null));
+	})
+}
+
+func TestTsconfigJsonOverrideMissing(t *testing.T) {
+	tsconfig_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/Users/user/project/src/app/entry.ts": `
+				import 'foo'
+			`,
+			"/Users/user/project/src/foo-bad.ts": `
+				console.log('bad')
+			`,
+			"/Users/user/project/src/tsconfig.json": `
+				{
+					"compilerOptions": {
+						"baseUrl": ".",
+						"paths": {
+							"foo": ["./foo-bad.ts"]
+						}
+					}
+				}
+			`,
+			"/Users/user/project/other/foo-good.ts": `
+				console.log('good')
+			`,
+			"/Users/user/project/other/config-for-ts.json": `
+				{
+					"compilerOptions": {
+						"baseUrl": ".",
+						"paths": {
+							"foo": ["./foo-good.ts"]
+						}
+					}
+				}
+			`,
+		},
+		entryPaths: []string{"/Users/user/project/src/app/entry.ts"},
+		options: config.Options{
+			Mode:             config.ModeBundle,
+			AbsOutputFile:    "/Users/user/project/out.js",
+			TsConfigOverride: "/Users/user/project/other/config-for-ts.json",
+		},
+	})
+}
+
+func TestTsconfigJsonOverrideNodeModules(t *testing.T) {
+	tsconfig_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/Users/user/project/src/app/entry.ts": `
+				import 'foo'
+			`,
+			"/Users/user/project/src/node_modules/foo/index.js": `
+				console.log('default')
+			`,
+			"/Users/user/project/src/foo-bad.ts": `
+				console.log('bad')
+			`,
+			"/Users/user/project/src/tsconfig.json": `
+				{
+					"compilerOptions": {
+						"baseUrl": ".",
+						"paths": {
+							"foo": ["./foo-bad.ts"]
+						}
+					}
+				}
+			`,
+			"/Users/user/project/other/foo-good.ts": `
+				console.log('good')
+			`,
+			"/Users/user/project/other/config-for-ts.json": `
+				{
+					"compilerOptions": {
+						"baseUrl": ".",
+						"paths": {
+							"foo": ["./foo-good.ts"]
+						}
+					}
+				}
+			`,
+		},
+		entryPaths: []string{"/Users/user/project/src/app/entry.ts"},
+		options: config.Options{
+			Mode:             config.ModeBundle,
+			AbsOutputFile:    "/Users/user/project/out.js",
+			TsConfigOverride: "/Users/user/project/other/config-for-ts.json",
+		},
+	})
+}
+
+func TestTsconfigJsonOverrideInvalid(t *testing.T) {
+	tsconfig_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.ts": ``,
+		},
+		entryPaths: []string{"/entry.ts"},
+		options: config.Options{
+			Mode:             config.ModeBundle,
+			AbsOutputFile:    "/out.js",
+			TsConfigOverride: "/this/file/doesn't/exist/tsconfig.json",
+		},
+		expectedScanLog: "error: Cannot find tsconfig file \"/this/file/doesn't/exist/tsconfig.json\"\n",
+	})
+}
+
+func TestTsconfigJsonNodeModulesImplicitFile(t *testing.T) {
+	tsconfig_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/Users/user/project/src/app/entry.tsx": `
+				console.log(<div/>)
+			`,
+			"/Users/user/project/src/tsconfig.json": `
+				{
+					"extends": "foo"
+				}
+			`,
+			"/Users/user/project/src/node_modules/foo/tsconfig.json": `
+				{
+					"compilerOptions": {
+						"jsx": "react",
+						"jsxFactory": "worked"
+					}
+				}
+			`,
+		},
+		entryPaths: []string{"/Users/user/project/src/app/entry.tsx"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/Users/user/project/out.js",
+		},
+	})
+}
+
+func TestTsconfigWarningsInsideNodeModules(t *testing.T) {
+	tsconfig_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/Users/user/project/src/entry.tsx": `
+				import "./foo"
+				import "bar"
+			`,
+
+			"/Users/user/project/src/foo/tsconfig.json": `{ "extends": "extends for foo" }`,
+			"/Users/user/project/src/foo/index.js":      ``,
+
+			"/Users/user/project/src/node_modules/bar/tsconfig.json": `{ "extends": "extends for bar" }`,
+			"/Users/user/project/src/node_modules/bar/index.js":      ``,
+		},
+		entryPaths: []string{"/Users/user/project/src/entry.tsx"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/Users/user/project/out.js",
+		},
+		expectedScanLog: `/Users/user/project/src/foo/tsconfig.json: warning: Cannot find base config file "extends for foo"
 `,
-		},
 	})
 }
