@@ -17,8 +17,7 @@
 //     package main
 //
 //     import (
-//         "fmt"
-//         "io/ioutil"
+//         "os"
 //
 //         "github.com/evanw/esbuild/pkg/api"
 //     )
@@ -113,6 +112,7 @@ const (
 	LoaderDataURL
 	LoaderFile
 	LoaderBinary
+	LoaderCSS
 )
 
 type Platform uint8
@@ -179,6 +179,7 @@ const (
 
 type StrictOptions struct {
 	NullishCoalescing bool
+	OptionalChaining  bool
 	ClassFields       bool
 }
 
@@ -214,9 +215,11 @@ type BuildOptions struct {
 	Platform          Platform
 	Format            Format
 	Externals         []string
+	MainFields        []string
 	Loaders           map[string]Loader
 	ResolveExtensions []string
 	Tsconfig          string
+	OutExtensions     map[string]string
 
 	EntryPoints []string
 	Stdin       *StdinOptions
@@ -254,10 +257,12 @@ type TransformOptions struct {
 	ErrorLimit int
 	LogLevel   LogLevel
 
-	Sourcemap SourceMap
-	Target    Target
-	Engines   []Engine
-	Strict    StrictOptions
+	Sourcemap  SourceMap
+	Target     Target
+	Format     Format
+	GlobalName string
+	Engines    []Engine
+	Strict     StrictOptions
 
 	MinifyWhitespace  bool
 	MinifyIdentifiers bool
