@@ -8,15 +8,16 @@ import (
 	"regexp"
 	"strconv"
 	"strings"
+	"sync"
 
-	"github.com/evanw/esbuild/internal/ast"
 	"github.com/evanw/esbuild/internal/bundler"
 	"github.com/evanw/esbuild/internal/compat"
 	"github.com/evanw/esbuild/internal/config"
 	"github.com/evanw/esbuild/internal/fs"
-	"github.com/evanw/esbuild/internal/lexer"
-	"github.com/evanw/esbuild/internal/logging"
-	"github.com/evanw/esbuild/internal/parser"
+	"github.com/evanw/esbuild/internal/js_ast"
+	"github.com/evanw/esbuild/internal/js_lexer"
+	"github.com/evanw/esbuild/internal/js_parser"
+	"github.com/evanw/esbuild/internal/logger"
 	"github.com/evanw/esbuild/internal/resolver"
 )
 
@@ -61,27 +62,27 @@ func validateSourceMap(value SourceMap) config.SourceMap {
 	}
 }
 
-func validateColor(value StderrColor) logging.StderrColor {
+func validateColor(value StderrColor) logger.StderrColor {
 	switch value {
 	case ColorIfTerminal:
-		return logging.ColorIfTerminal
+		return logger.ColorIfTerminal
 	case ColorNever:
-		return logging.ColorNever
+		return logger.ColorNever
 	case ColorAlways:
-		return logging.ColorAlways
+		return logger.ColorAlways
 	default:
 		panic("Invalid color")
 	}
 }
 
-func validateLogLevel(value LogLevel) logging.LogLevel {
+func validateLogLevel(value LogLevel) logger.LogLevel {
 	switch value {
 	case LogLevelInfo:
-		return logging.LevelInfo
+		return logger.LevelInfo
 	case LogLevelWarning:
-		return logging.LevelWarning
+		return logger.LevelWarning
 	case LogLevelError:
-		return logging.LevelError
+		return logger.LevelError
 	default:
 		panic("Invalid log level")
 	}
@@ -90,6 +91,7 @@ func validateLogLevel(value LogLevel) logging.LogLevel {
 func validateStrict(value StrictOptions) config.StrictOptions {
 	return config.StrictOptions{
 		NullishCoalescing: value.NullishCoalescing,
+		OptionalChaining:  value.OptionalChaining,
 		ClassFields:       value.ClassFields,
 	}
 }
@@ -116,6 +118,8 @@ func validateLoader(value Loader) config.Loader {
 		return config.LoaderFile
 	case LoaderBinary:
 		return config.LoaderBinary
+	case LoaderCSS:
+		return config.LoaderCSS
 	default:
 		panic("Invalid loader")
 	}
@@ -142,7 +146,7 @@ func validateEngine(value EngineName) compat.Engine {
 
 var versionRegex = regexp.MustCompile(`^([0-9]+)(?:\.([0-9]+))?(?:\.([0-9]+))?$`)
 
-func validateFeatures(log logging.Log, target Target, engines []Engine) compat.Feature {
+func validateFeatures(log logger.Log, target Target, engines []Engine) (compat.JSFeature, compat.CSSFeature) {
 	constraints := make(map[compat.Engine][]int)
 
 	switch target {
@@ -195,13 +199,13 @@ func validateFeatures(log logging.Log, target Target, engines []Engine) compat.F
 			}
 		}
 
-		log.AddError(nil, ast.Loc{}, fmt.Sprintf("Invalid version: %q", engine.Version))
+		log.AddError(nil, logger.Loc{}, fmt.Sprintf("Invalid version: %q", engine.Version))
 	}
 
-	return compat.UnsupportedFeatures(constraints)
+	return compat.UnsupportedJSFeatures(constraints), compat.UnsupportedCSSFeatures(constraints)
 }
 
-func validateExternals(log logging.Log, fs fs.FS, paths []string) config.ExternalModules {
+func validateExternals(log logger.Log, fs fs.FS, paths []string) config.ExternalModules {
 	result := config.ExternalModules{
 		NodeModules: make(map[string]bool),
 		AbsPaths:    make(map[string]bool),
@@ -216,24 +220,28 @@ func validateExternals(log logging.Log, fs fs.FS, paths []string) config.Externa
 	return result
 }
 
-func validateResolveExtensions(log logging.Log, order []string) []string {
+func isValidExtension(ext string) bool {
+	return len(ext) >= 2 && ext[0] == '.' && ext[len(ext)-1] != '.'
+}
+
+func validateResolveExtensions(log logger.Log, order []string) []string {
 	if order == nil {
-		return []string{".tsx", ".ts", ".jsx", ".mjs", ".cjs", ".js", ".json"}
+		return []string{".tsx", ".ts", ".jsx", ".mjs", ".cjs", ".js", ".css", ".json"}
 	}
 	for _, ext := range order {
-		if len(ext) < 2 || ext[0] != '.' {
-			log.AddError(nil, ast.Loc{}, fmt.Sprintf("Invalid file extension: %q", ext))
+		if !isValidExtension(ext) {
+			log.AddError(nil, logger.Loc{}, fmt.Sprintf("Invalid file extension: %q", ext))
 		}
 	}
 	return order
 }
 
-func validateLoaders(log logging.Log, loaders map[string]Loader) map[string]config.Loader {
+func validateLoaders(log logger.Log, loaders map[string]Loader) map[string]config.Loader {
 	result := bundler.DefaultExtensionToLoaderMap()
 	if loaders != nil {
 		for ext, loader := range loaders {
-			if len(ext) < 2 || ext[0] != '.' || ext[len(ext)-1] == '.' {
-				log.AddError(nil, ast.Loc{}, fmt.Sprintf("Invalid file extension: %q", ext))
+			if !isValidExtension(ext) {
+				log.AddError(nil, logger.Loc{}, fmt.Sprintf("Invalid file extension: %q", ext))
 			}
 			result[ext] = validateLoader(loader)
 		}
@@ -241,21 +249,21 @@ func validateLoaders(log logging.Log, loaders map[string]Loader) map[string]conf
 	return result
 }
 
-func validateJSX(log logging.Log, text string, name string) []string {
+func validateJSX(log logger.Log, text string, name string) []string {
 	if text == "" {
 		return nil
 	}
 	parts := strings.Split(text, ".")
 	for _, part := range parts {
-		if !lexer.IsIdentifier(part) {
-			log.AddError(nil, ast.Loc{}, fmt.Sprintf("Invalid JSX %s: %q", name, text))
+		if !js_lexer.IsIdentifier(part) {
+			log.AddError(nil, logger.Loc{}, fmt.Sprintf("Invalid JSX %s: %q", name, text))
 			return nil
 		}
 	}
 	return parts
 }
 
-func validateDefines(log logging.Log, defines map[string]string, pureFns []string) *config.ProcessedDefines {
+func validateDefines(log logger.Log, defines map[string]string, pureFns []string) *config.ProcessedDefines {
 	if len(defines) == 0 && len(pureFns) == 0 {
 		return nil
 	}
@@ -265,19 +273,19 @@ func validateDefines(log logging.Log, defines map[string]string, pureFns []strin
 	for key, value := range defines {
 		// The key must be a dot-separated identifier list
 		for _, part := range strings.Split(key, ".") {
-			if !lexer.IsIdentifier(part) {
-				log.AddError(nil, ast.Loc{}, fmt.Sprintf("Invalid define key: %q", key))
+			if !js_lexer.IsIdentifier(part) {
+				log.AddError(nil, logger.Loc{}, fmt.Sprintf("Invalid define key: %q", key))
 				continue
 			}
 		}
 
 		// Allow substituting for an identifier
-		if lexer.IsIdentifier(value) {
-			if _, ok := lexer.Keywords()[value]; !ok {
+		if js_lexer.IsIdentifier(value) {
+			if _, ok := js_lexer.Keywords[value]; !ok {
 				name := value // The closure must close over a variable inside the loop
 				rawDefines[key] = config.DefineData{
-					DefineFunc: func(findSymbol config.FindSymbol) ast.E {
-						return &ast.EIdentifier{Ref: findSymbol(name)}
+					DefineFunc: func(loc logger.Loc, findSymbol config.FindSymbol) js_ast.E {
+						return &js_ast.EIdentifier{Ref: findSymbol(loc, name)}
 					},
 				}
 				continue
@@ -285,26 +293,26 @@ func validateDefines(log logging.Log, defines map[string]string, pureFns []strin
 		}
 
 		// Parse the value as JSON
-		source := logging.Source{Contents: value}
-		expr, ok := parser.ParseJSON(logging.NewDeferLog(), source, parser.ParseJSONOptions{})
+		source := logger.Source{Contents: value}
+		expr, ok := js_parser.ParseJSON(logger.NewDeferLog(), source, js_parser.ParseJSONOptions{})
 		if !ok {
-			log.AddError(nil, ast.Loc{}, fmt.Sprintf("Invalid define value: %q", value))
+			log.AddError(nil, logger.Loc{}, fmt.Sprintf("Invalid define value: %q", value))
 			continue
 		}
 
 		// Only allow atoms for now
 		var fn config.DefineFunc
 		switch e := expr.Data.(type) {
-		case *ast.ENull:
-			fn = func(config.FindSymbol) ast.E { return &ast.ENull{} }
-		case *ast.EBoolean:
-			fn = func(config.FindSymbol) ast.E { return &ast.EBoolean{Value: e.Value} }
-		case *ast.EString:
-			fn = func(config.FindSymbol) ast.E { return &ast.EString{Value: e.Value} }
-		case *ast.ENumber:
-			fn = func(config.FindSymbol) ast.E { return &ast.ENumber{Value: e.Value} }
+		case *js_ast.ENull:
+			fn = func(logger.Loc, config.FindSymbol) js_ast.E { return &js_ast.ENull{} }
+		case *js_ast.EBoolean:
+			fn = func(logger.Loc, config.FindSymbol) js_ast.E { return &js_ast.EBoolean{Value: e.Value} }
+		case *js_ast.EString:
+			fn = func(logger.Loc, config.FindSymbol) js_ast.E { return &js_ast.EString{Value: e.Value} }
+		case *js_ast.ENumber:
+			fn = func(logger.Loc, config.FindSymbol) js_ast.E { return &js_ast.ENumber{Value: e.Value} }
 		default:
-			log.AddError(nil, ast.Loc{}, fmt.Sprintf("Invalid define value: %q", value))
+			log.AddError(nil, logger.Loc{}, fmt.Sprintf("Invalid define value: %q", value))
 			continue
 		}
 
@@ -314,8 +322,8 @@ func validateDefines(log logging.Log, defines map[string]string, pureFns []strin
 	for _, key := range pureFns {
 		// The key must be a dot-separated identifier list
 		for _, part := range strings.Split(key, ".") {
-			if !lexer.IsIdentifier(part) {
-				log.AddError(nil, ast.Loc{}, fmt.Sprintf("Invalid pure function: %q", key))
+			if !js_lexer.IsIdentifier(part) {
+				log.AddError(nil, logger.Loc{}, fmt.Sprintf("Invalid pure function: %q", key))
 				continue
 			}
 		}
@@ -332,18 +340,32 @@ func validateDefines(log logging.Log, defines map[string]string, pureFns []strin
 	return &processed
 }
 
-func validatePath(log logging.Log, fs fs.FS, relPath string) string {
+func validatePath(log logger.Log, fs fs.FS, relPath string) string {
 	if relPath == "" {
 		return ""
 	}
 	absPath, ok := fs.Abs(relPath)
 	if !ok {
-		log.AddError(nil, ast.Loc{}, fmt.Sprintf("Invalid path: %s", relPath))
+		log.AddError(nil, logger.Loc{}, fmt.Sprintf("Invalid path: %s", relPath))
 	}
 	return absPath
 }
 
-func messagesOfKind(kind logging.MsgKind, msgs []logging.Msg) []Message {
+func validateOutputExtensions(log logger.Log, outExtensions map[string]string) map[string]string {
+	result := make(map[string]string)
+	for key, value := range outExtensions {
+		if key != ".js" {
+			log.AddError(nil, logger.Loc{}, fmt.Sprintf("Invalid output extension: %q (valid: .js)", key))
+		}
+		if !isValidExtension(value) {
+			log.AddError(nil, logger.Loc{}, fmt.Sprintf("Invalid output extension: %q", value))
+		}
+		result[key] = value
+	}
+	return result
+}
+
+func messagesOfKind(kind logger.MsgKind, msgs []logger.Msg) []Message {
 	var filtered []Message
 	for _, msg := range msgs {
 		if msg.Kind == kind {
@@ -373,11 +395,11 @@ func messagesOfKind(kind logging.MsgKind, msgs []logging.Msg) []Message {
 // Build API
 
 func buildImpl(buildOpts BuildOptions) BuildResult {
-	var log logging.Log
+	var log logger.Log
 	if buildOpts.LogLevel == LogLevelSilent {
-		log = logging.NewDeferLog()
+		log = logger.NewDeferLog()
 	} else {
-		log = logging.NewStderrLog(logging.StderrOptions{
+		log = logger.NewStderrLog(logger.StderrOptions{
 			IncludeSource: true,
 			ErrorLimit:    buildOpts.ErrorLimit,
 			Color:         validateColor(buildOpts.Color),
@@ -387,9 +409,11 @@ func buildImpl(buildOpts BuildOptions) BuildResult {
 
 	// Convert and validate the buildOpts
 	realFS := fs.RealFS()
+	jsFeatures, cssFeatures := validateFeatures(log, buildOpts.Target, buildOpts.Engines)
 	options := config.Options{
-		UnsupportedFeatures: validateFeatures(log, buildOpts.Target, buildOpts.Engines),
-		Strict:              validateStrict(buildOpts.Strict),
+		UnsupportedJSFeatures:  jsFeatures,
+		UnsupportedCSSFeatures: cssFeatures,
+		Strict:                 validateStrict(buildOpts.Strict),
 		JSX: config.JSXOptions{
 			Factory:  validateJSX(log, buildOpts.JSXFactory, "factory"),
 			Fragment: validateJSX(log, buildOpts.JSXFragment, "fragment"),
@@ -401,16 +425,17 @@ func buildImpl(buildOpts BuildOptions) BuildResult {
 		RemoveWhitespace:  buildOpts.MinifyWhitespace,
 		MinifyIdentifiers: buildOpts.MinifyIdentifiers,
 		ModuleName:        buildOpts.GlobalName,
-		IsBundling:        buildOpts.Bundle,
 		CodeSplitting:     buildOpts.Splitting,
 		OutputFormat:      validateFormat(buildOpts.Format),
 		AbsOutputFile:     validatePath(log, realFS, buildOpts.Outfile),
 		AbsOutputDir:      validatePath(log, realFS, buildOpts.Outdir),
 		AbsMetadataFile:   validatePath(log, realFS, buildOpts.Metafile),
+		OutputExtensions:  validateOutputExtensions(log, buildOpts.OutExtensions),
 		ExtensionToLoader: validateLoaders(log, buildOpts.Loaders),
 		ExtensionOrder:    validateResolveExtensions(log, buildOpts.ResolveExtensions),
 		ExternalModules:   validateExternals(log, realFS, buildOpts.Externals),
 		TsConfigOverride:  validatePath(log, realFS, buildOpts.Tsconfig),
+		MainFields:        buildOpts.MainFields,
 	}
 	entryPaths := make([]string, len(buildOpts.EntryPoints))
 	for i, entryPoint := range buildOpts.EntryPoints {
@@ -428,13 +453,13 @@ func buildImpl(buildOpts BuildOptions) BuildResult {
 	}
 
 	if options.AbsOutputDir == "" && entryPathCount > 1 {
-		log.AddError(nil, ast.Loc{},
+		log.AddError(nil, logger.Loc{},
 			"Must use \"outdir\" when there are multiple input files")
 	} else if options.AbsOutputDir == "" && options.CodeSplitting {
-		log.AddError(nil, ast.Loc{},
+		log.AddError(nil, logger.Loc{},
 			"Must use \"outdir\" when code splitting is enabled")
 	} else if options.AbsOutputFile != "" && options.AbsOutputDir != "" {
-		log.AddError(nil, ast.Loc{}, "Cannot use both \"outfile\" and \"outdir\"")
+		log.AddError(nil, logger.Loc{}, "Cannot use both \"outfile\" and \"outdir\"")
 	} else if options.AbsOutputFile != "" {
 		// If the output file is specified, use it to derive the output directory
 		options.AbsOutputDir = realFS.Dir(options.AbsOutputFile)
@@ -443,14 +468,14 @@ func buildImpl(buildOpts BuildOptions) BuildResult {
 
 		// Forbid certain features when writing to stdout
 		if options.SourceMap != config.SourceMapNone && options.SourceMap != config.SourceMapInline {
-			log.AddError(nil, ast.Loc{}, "Cannot use an external source map without an output path")
+			log.AddError(nil, logger.Loc{}, "Cannot use an external source map without an output path")
 		}
 		if options.AbsMetadataFile != "" {
-			log.AddError(nil, ast.Loc{}, "Cannot use \"metafile\" without an output path")
+			log.AddError(nil, logger.Loc{}, "Cannot use \"metafile\" without an output path")
 		}
 		for _, loader := range options.ExtensionToLoader {
 			if loader == config.LoaderFile {
-				log.AddError(nil, ast.Loc{}, "Cannot use the \"file\" loader without an output path")
+				log.AddError(nil, logger.Loc{}, "Cannot use the \"file\" loader without an output path")
 				break
 			}
 		}
@@ -460,13 +485,10 @@ func buildImpl(buildOpts BuildOptions) BuildResult {
 		options.AbsOutputDir = realFS.Cwd()
 	}
 
-	if !options.IsBundling {
+	if !buildOpts.Bundle {
 		// Disallow bundle-only options when not bundling
-		if options.OutputFormat != config.FormatPreserve {
-			log.AddError(nil, ast.Loc{}, "Cannot use \"format\" without \"bundle\"")
-		}
 		if len(options.ExternalModules.NodeModules) > 0 || len(options.ExternalModules.AbsPaths) > 0 {
-			log.AddError(nil, ast.Loc{}, "Cannot use \"external\" without \"bundle\"")
+			log.AddError(nil, logger.Loc{}, "Cannot use \"external\" without \"bundle\"")
 		}
 	} else if options.OutputFormat == config.FormatPreserve {
 		// If the format isn't specified, set the default format using the platform
@@ -478,9 +500,16 @@ func buildImpl(buildOpts BuildOptions) BuildResult {
 		}
 	}
 
+	// Set the output mode using other settings
+	if buildOpts.Bundle {
+		options.Mode = config.ModeBundle
+	} else if options.OutputFormat != config.FormatPreserve {
+		options.Mode = config.ModeConvertFormat
+	}
+
 	// Code splitting is experimental and currently only enabled for ES6 modules
 	if options.CodeSplitting && options.OutputFormat != config.FormatESModule {
-		log.AddError(nil, ast.Loc{}, "Splitting currently only works with the \"esm\" format")
+		log.AddError(nil, logger.Loc{}, "Splitting currently only works with the \"esm\" format")
 	}
 
 	var outputFiles []OutputFile
@@ -496,37 +525,55 @@ func buildImpl(buildOpts BuildOptions) BuildResult {
 			// Compile the bundle
 			results := bundle.Compile(log, options)
 
-			// Return the results
-			outputFiles = make([]OutputFile, len(results))
-			for i, result := range results {
-				if options.WriteToStdout {
-					result.AbsPath = "<stdout>"
-				}
-				outputFiles[i] = OutputFile{
-					Path:     result.AbsPath,
-					Contents: result.Contents,
-				}
-			}
-
-			if buildOpts.Write {
-				// Special-case writing to stdout
-				if options.WriteToStdout {
-					if len(outputFiles) != 1 {
-						log.AddError(nil, ast.Loc{}, fmt.Sprintf(
-							"Internal error: did not expect to generate %d files when writing to stdout", len(outputFiles)))
-					} else if _, err := os.Stdout.Write(outputFiles[0].Contents); err != nil {
-						log.AddError(nil, ast.Loc{}, fmt.Sprintf(
-							"Failed to write to stdout: %s", err.Error()))
-					}
-				} else {
-					for _, outputFile := range outputFiles {
-						if err := os.MkdirAll(filepath.Dir(outputFile.Path), 0755); err != nil {
-							log.AddError(nil, ast.Loc{}, fmt.Sprintf(
-								"Failed to create output directory: %s", err.Error()))
-						} else if err := ioutil.WriteFile(outputFile.Path, outputFile.Contents, 0644); err != nil {
-							log.AddError(nil, ast.Loc{}, fmt.Sprintf(
-								"Failed to write to output file: %s", err.Error()))
+			// Stop now if there were errors
+			if !log.HasErrors() {
+				if buildOpts.Write {
+					// Special-case writing to stdout
+					if options.WriteToStdout {
+						if len(results) != 1 {
+							log.AddError(nil, logger.Loc{}, fmt.Sprintf(
+								"Internal error: did not expect to generate %d files when writing to stdout", len(results)))
+						} else if _, err := os.Stdout.Write(results[0].Contents); err != nil {
+							log.AddError(nil, logger.Loc{}, fmt.Sprintf(
+								"Failed to write to stdout: %s", err.Error()))
 						}
+					} else {
+						// Write out files in parallel
+						waitGroup := sync.WaitGroup{}
+						waitGroup.Add(len(results))
+						for _, result := range results {
+							go func(result bundler.OutputFile) {
+								fs.BeforeFileOpen()
+								defer fs.AfterFileClose()
+								if err := os.MkdirAll(filepath.Dir(result.AbsPath), 0755); err != nil {
+									log.AddError(nil, logger.Loc{}, fmt.Sprintf(
+										"Failed to create output directory: %s", err.Error()))
+								} else {
+									var mode os.FileMode = 0644
+									if result.IsExecutable {
+										mode = 0755
+									}
+									if err := ioutil.WriteFile(result.AbsPath, result.Contents, mode); err != nil {
+										log.AddError(nil, logger.Loc{}, fmt.Sprintf(
+											"Failed to write to output file: %s", err.Error()))
+									}
+								}
+								waitGroup.Done()
+							}(result)
+						}
+						waitGroup.Wait()
+					}
+				}
+
+				// Return the results
+				outputFiles = make([]OutputFile, len(results))
+				for i, result := range results {
+					if options.WriteToStdout {
+						result.AbsPath = "<stdout>"
+					}
+					outputFiles[i] = OutputFile{
+						Path:     result.AbsPath,
+						Contents: result.Contents,
 					}
 				}
 			}
@@ -535,8 +582,8 @@ func buildImpl(buildOpts BuildOptions) BuildResult {
 
 	msgs := log.Done()
 	return BuildResult{
-		Errors:      messagesOfKind(logging.Error, msgs),
-		Warnings:    messagesOfKind(logging.Warning, msgs),
+		Errors:      messagesOfKind(logger.Error, msgs),
+		Warnings:    messagesOfKind(logger.Warning, msgs),
 		OutputFiles: outputFiles,
 	}
 }
@@ -545,11 +592,11 @@ func buildImpl(buildOpts BuildOptions) BuildResult {
 // Transform API
 
 func transformImpl(input string, transformOpts TransformOptions) TransformResult {
-	var log logging.Log
+	var log logger.Log
 	if transformOpts.LogLevel == LogLevelSilent {
-		log = logging.NewDeferLog()
+		log = logger.NewDeferLog()
 	} else {
-		log = logging.NewStderrLog(logging.StderrOptions{
+		log = logger.NewStderrLog(logger.StderrOptions{
 			IncludeSource: true,
 			ErrorLimit:    transformOpts.ErrorLimit,
 			Color:         validateColor(transformOpts.Color),
@@ -558,15 +605,19 @@ func transformImpl(input string, transformOpts TransformOptions) TransformResult
 	}
 
 	// Convert and validate the transformOpts
+	jsFeatures, cssFeatures := validateFeatures(log, transformOpts.Target, transformOpts.Engines)
 	options := config.Options{
-		UnsupportedFeatures: validateFeatures(log, transformOpts.Target, transformOpts.Engines),
-		Strict:              validateStrict(transformOpts.Strict),
+		UnsupportedJSFeatures:  jsFeatures,
+		UnsupportedCSSFeatures: cssFeatures,
+		Strict:                 validateStrict(transformOpts.Strict),
 		JSX: config.JSXOptions{
 			Factory:  validateJSX(log, transformOpts.JSXFactory, "factory"),
 			Fragment: validateJSX(log, transformOpts.JSXFragment, "fragment"),
 		},
 		Defines:           validateDefines(log, transformOpts.Defines, transformOpts.PureFunctions),
 		SourceMap:         validateSourceMap(transformOpts.Sourcemap),
+		OutputFormat:      validateFormat(transformOpts.Format),
+		ModuleName:        transformOpts.GlobalName,
 		MangleSyntax:      transformOpts.MinifySyntax,
 		RemoveWhitespace:  transformOpts.MinifyWhitespace,
 		MinifyIdentifiers: transformOpts.MinifyIdentifiers,
@@ -579,13 +630,18 @@ func transformImpl(input string, transformOpts TransformOptions) TransformResult
 	}
 	if options.SourceMap == config.SourceMapLinkedWithComment {
 		// Linked source maps don't make sense because there's no output file name
-		log.AddError(nil, ast.Loc{}, "Cannot transform with linked source maps")
+		log.AddError(nil, logger.Loc{}, "Cannot transform with linked source maps")
 	}
 	if options.SourceMap != config.SourceMapNone && options.Stdin.SourceFile == "" {
-		log.AddError(nil, ast.Loc{},
+		log.AddError(nil, logger.Loc{},
 			"Must use \"sourcefile\" with \"sourcemap\" to set the original file name")
 	}
 
+	// Set the output mode using other settings
+	if options.OutputFormat != config.FormatPreserve {
+		options.Mode = config.ModeConvertFormat
+	}
+
 	var results []bundler.OutputFile
 
 	// Stop now if there were errors
@@ -620,8 +676,8 @@ func transformImpl(input string, transformOpts TransformOptions) TransformResult
 
 	msgs := log.Done()
 	return TransformResult{
-		Errors:      messagesOfKind(logging.Error, msgs),
-		Warnings:    messagesOfKind(logging.Warning, msgs),
+		Errors:      messagesOfKind(logger.Error, msgs),
+		Warnings:    messagesOfKind(logger.Warning, msgs),
 		JS:          js,
 		JSSourceMap: jsSourceMap,
 	}
