@@ -8,7 +8,7 @@ import (
 	"strconv"
 	"strings"
 
-	"github.com/evanw/esbuild/internal/logging"
+	"github.com/evanw/esbuild/internal/logger"
 	"github.com/evanw/esbuild/pkg/api"
 )
 
@@ -106,8 +106,15 @@ func parseOptionsImpl(osArgs []string, buildOpts *api.BuildOptions, transformOpt
 		case strings.HasPrefix(arg, "--resolve-extensions=") && buildOpts != nil:
 			buildOpts.ResolveExtensions = strings.Split(arg[len("--resolve-extensions="):], ",")
 
-		case strings.HasPrefix(arg, "--global-name=") && buildOpts != nil:
-			buildOpts.GlobalName = arg[len("--global-name="):]
+		case strings.HasPrefix(arg, "--main-fields=") && buildOpts != nil:
+			buildOpts.MainFields = strings.Split(arg[len("--main-fields="):], ",")
+
+		case strings.HasPrefix(arg, "--global-name="):
+			if buildOpts != nil {
+				buildOpts.GlobalName = arg[len("--global-name="):]
+			} else {
+				transformOpts.GlobalName = arg[len("--global-name="):]
+			}
 
 		case strings.HasPrefix(arg, "--metafile=") && buildOpts != nil:
 			buildOpts.Metafile = arg[len("--metafile="):]
@@ -185,9 +192,21 @@ func parseOptionsImpl(osArgs []string, buildOpts *api.BuildOptions, transformOpt
 				transformOpts.Engines = engines
 			}
 
+		case strings.HasPrefix(arg, "--out-extension:") && buildOpts != nil:
+			value := arg[len("--out-extension:"):]
+			equals := strings.IndexByte(value, '=')
+			if equals == -1 {
+				return fmt.Errorf("Missing \"=\": %q", value)
+			}
+			if buildOpts.OutExtensions == nil {
+				buildOpts.OutExtensions = make(map[string]string)
+			}
+			buildOpts.OutExtensions[value[:equals]] = value[equals+1:]
+
 		case arg == "--strict":
 			value := api.StrictOptions{
 				NullishCoalescing: true,
+				OptionalChaining:  true,
 				ClassFields:       true,
 			}
 			if buildOpts != nil {
@@ -207,10 +226,12 @@ func parseOptionsImpl(osArgs []string, buildOpts *api.BuildOptions, transformOpt
 			switch name {
 			case "nullish-coalescing":
 				value.NullishCoalescing = true
+			case "optional-chaining":
+				value.OptionalChaining = true
 			case "class-fields":
 				value.ClassFields = true
 			default:
-				return fmt.Errorf("Invalid strict value: %q (valid: nullish-coalescing, class-fields)", name)
+				return fmt.Errorf("Invalid strict value: %q (valid: nullish-coalescing, optional-chaining, class-fields)", name)
 			}
 
 		case strings.HasPrefix(arg, "--platform=") && buildOpts != nil:
@@ -224,15 +245,27 @@ func parseOptionsImpl(osArgs []string, buildOpts *api.BuildOptions, transformOpt
 				return fmt.Errorf("Invalid platform: %q (valid: browser, node)", value)
 			}
 
-		case strings.HasPrefix(arg, "--format=") && buildOpts != nil:
+		case strings.HasPrefix(arg, "--format="):
 			value := arg[len("--format="):]
 			switch value {
 			case "iife":
-				buildOpts.Format = api.FormatIIFE
+				if buildOpts != nil {
+					buildOpts.Format = api.FormatIIFE
+				} else {
+					transformOpts.Format = api.FormatIIFE
+				}
 			case "cjs":
-				buildOpts.Format = api.FormatCommonJS
+				if buildOpts != nil {
+					buildOpts.Format = api.FormatCommonJS
+				} else {
+					transformOpts.Format = api.FormatCommonJS
+				}
 			case "esm":
-				buildOpts.Format = api.FormatESModule
+				if buildOpts != nil {
+					buildOpts.Format = api.FormatESModule
+				} else {
+					transformOpts.Format = api.FormatESModule
+				}
 			default:
 				return fmt.Errorf("Invalid format: %q (valid: iife, cjs, esm)", value)
 			}
@@ -391,6 +424,8 @@ func parseLoader(text string) (api.Loader, error) {
 		return api.LoaderTS, nil
 	case "tsx":
 		return api.LoaderTSX, nil
+	case "css":
+		return api.LoaderCSS, nil
 	case "json":
 		return api.LoaderJSON, nil
 	case "text":
@@ -405,7 +440,7 @@ func parseLoader(text string) (api.Loader, error) {
 		return api.LoaderBinary, nil
 	default:
 		return 0, fmt.Errorf("Invalid loader: %q (valid: "+
-			"js, jsx, ts, tsx, json, text, base64, dataurl, file, binary)", text)
+			"js, jsx, ts, tsx, css, json, text, base64, dataurl, file, binary)", text)
 	}
 }
 
@@ -458,17 +493,17 @@ func runImpl(osArgs []string) int {
 			}
 			bytes, err := ioutil.ReadAll(os.Stdin)
 			if err != nil {
-				logging.PrintErrorToStderr(osArgs, fmt.Sprintf(
+				logger.PrintErrorToStderr(osArgs, fmt.Sprintf(
 					"Could not read from stdin: %s", err.Error()))
 				return 1
 			}
 			buildOptions.Stdin.Contents = string(bytes)
 		} else if buildOptions.Stdin != nil {
 			if buildOptions.Stdin.Sourcefile != "" {
-				logging.PrintErrorToStderr(osArgs,
+				logger.PrintErrorToStderr(osArgs,
 					"\"sourcefile\" only applies when reading from stdin")
 			} else {
-				logging.PrintErrorToStderr(osArgs,
+				logger.PrintErrorToStderr(osArgs,
 					"\"loader\" without extension only applies when reading from stdin")
 			}
 			return 1
@@ -484,7 +519,7 @@ func runImpl(osArgs []string) int {
 		// Read the input from stdin
 		bytes, err := ioutil.ReadAll(os.Stdin)
 		if err != nil {
-			logging.PrintErrorToStderr(osArgs, fmt.Sprintf(
+			logger.PrintErrorToStderr(osArgs, fmt.Sprintf(
 				"Could not read from stdin: %s", err.Error()))
 			return 1
 		}
@@ -499,7 +534,7 @@ func runImpl(osArgs []string) int {
 		os.Stdout.Write(result.JS)
 
 	case err != nil:
-		logging.PrintErrorToStderr(osArgs, err.Error())
+		logger.PrintErrorToStderr(osArgs, err.Error())
 		return 1
 	}
 
